/*
Bimetrics Binary PSO
  First version          : 2008-08-08
    Last update          : 
    email                : Maurice.Clerc@WriteMe.com
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/* WHERE IS IT COMING FROM?
  
	Since 2004, I tried a lot (I mean a LOT) of Binary PSOs.
	In this program, I keep only two quite different strategies, 
	which can be combined:

	Strategy 0 transforms each binary string into a real number, or 2 or 3, 
		depending on your computer (see parameter numSize). Therefore the optimisation
			is a 1D (or 2D, 3D) one, which is solved by using Standard PSO 2007.
				Pretty good on a lot of problems. Not that the way the binary string is transformed
		into a number is randomly modified before each run. After all, if the length
		of the binary string is N, there are N! possibilities, so it seems reasonable
		to not keep the same for all runs.

	Strategy 1 is simply
	"at random around the best known position in the social neighbourhood"

	The infonetwork (information links) is modified at random after 
	each unsuccessful iteration, as in Standard PSO 2007 
	(Particle Swarm Central, http://www.particleswarm.info)
*/  

 /* HOW TO USE IT
  1. Define the problem. See pbDefine(). 
		Of course, you may add your own problem in perf()

	2. Define the parameters
	As said, it is usually not really necessary, for the hard coded values 
 	(based on mathematical analysis's) are pretty good.
		Just see the parameter numSize when 
		 dimension > max_number_of_bits_of_an_integer_on_your_computer

	There is a special parameter called edgeListUse. What is it?
	If, for a given problem, the fitnesse evaluation is very costly, it is a good 
	idea to never re-evaluate a position (which is an edge of the {0,1}^D hypercube).
	Then, each known position and its fitness is stored in a list. 
	For a new position, after a move, the fitness is evaluated only if it is not
	already in the list. Note that the process "check if a position is in the list"
	may be quite time consuming. So, if the fitness evaluation is _not_ costly,
	it is not worth using this method. Better to re-evaluate.
	
 Have fun and keep me posted.

	Note: If your compiler uses a reasonably good pseudo random numbers generator
     you don't have to use KISS.
*/

/* SOME RESULTS, so that you can check your own implementation  
 
	Parameter edgeListUse is set to zero, numSize to 1
Problem_code	Name								Dimension	FEmax	runs
1							Goldberg						15				1000	100
2							Bipolar order 6			18				3000	100
3							Mühlenbein					15				3000	100
6							Zebra3							15				20000	100

Problem_code	Strategy	Success_probability

1							0					0.91
2							0					1.00				
3							0					0.99					
6							0					0.05

1							1				0.31
2							1				0.84
3							1				0.40
6							1				1

1							(0,1)		0.98
2							(0,1)		1.00
3							(0,1)		1.00
6							(0,1)		1.00	


On this small set of problems, the best method is the combination (0,1).

With edgeListUse=1, and edgeMax=10000 (time consuming option)
results are just slightly better, on the whole.

More results. Probability of success with strategy (0,1) 
---------------------------------------------------------
edgeListUse=0

 	Problem							Dimension			FEmax			runs	Strategy_(0,1)
1	Goldberg							30					20000			100			0.91
2	Bipolar order 6				30					20000			100			1.00
3	Mühlenbein						30					20000			100			0.94
6	Zebra3								30					20000			100			0.00 *
-1 Quadratic						100					20000			100			0.96
8	Multimodal 20 peaks		100					20000			100			0.16


 */

#include <stdio.h>
#include <stdlib.h>
#include "math.h"
#include <time.h>

#define	dMax 100 // Max number of dimensions of the search space
#define rMax 100 // Max number of runs for one problem
#define	sMax 100 // Max swarm size
#define peakMax 500
#define edgeMax 10000 // Max number of memorized edges (should be 2^DMax, 
// but this is usually impossible in practice)
#define qMax 100 // For quadratic problems
#define numSizeMax 3 // At most dMax/2.  For strategy 0

#define ulong unsigned long 	// For KISS pseudo random numbers generator
#define RAND_MAX_KISS ((unsigned long) 4294967295) // idem
					// You don't need KISS if you have a good rand() function
#define infinity 1.e+30;

// Structures

struct	bits {int size;int b[dMax];};
struct nums {int size;double num[numSizeMax];};
struct	position {struct bits bit; struct nums num;double f;};
struct velocity {struct bits bit; struct nums num;};

struct swarm
{
	int size;
	struct position x[sMax];
	struct velocity v[sMax];
	struct position xBest[sMax];
	struct position best;	
	int	links[sMax][sMax]; // Information links
	int strategy[sMax];
};

struct problem 
{
	int code;
	int D;
	double evalMax;
	double objective;
	int runs;
	int peaks; // Number of peaks for Multimodal random generator
	int landscape[peakMax][dMax]; // Locations of the peaks
	double density; // For  generated Quadratic problem
	int Q[qMax][qMax]; // Quadratic problem
};

struct parameter
{
	int method[3];			// method/strategy to use for moving the particles
	double methodProba[2]; // Probability of each method
	double pLink; 	// Probability to be an informant
	int S; 					// Swarm size
	double c0_1;
	double c0_2;
	int numSize;
	double c2_1; 	// Parameter 1 for method/strategy 2
	double c2_2;	// Parameter 2 for method/strategy 2
};

struct edgeList
{
	int size;
	struct position xBin[edgeMax];
	int times[edgeMax]; // Number of times the corresponding edge has been used
};


// Sub-programs
double	alea(double a,double b);
int		aleaInteger(int a,int b);
double	distance(struct position x1,struct position x2);

int edgeAdd(struct position xBin);
int edgeFind(struct position xBin);
void edgeListDisplay(struct edgeList edges);
int edgeNew(struct problem pb, int numSize); // Add a new edge to the list. Return its rank

void linksDisplay(int links[sMax][sMax],int s);
void linksInit(int links[sMax][sMax],int s, double pLink);

double bitToNum(struct bits x, int d,int code);
double bitToNum2(struct bits x, int d);
struct bits numToBit(double a, int D);
struct bits permutBit(int D);

struct parameter paramDefine(int D);

struct problem pbDefine();

double	perf(struct position x,struct problem pb);

void positionDisplay(struct position x);
struct position positionInit(struct problem pb,struct parameter param);
struct position positionUpdate
					(int s, struct problem pb, struct parameter param, struct swarm S);

struct position psoBin(struct problem pb, struct parameter param);
struct velocity velocityInit(struct problem pb, struct parameter param);
ulong	KissRand();
void	KissSeed(ulong seed);

// Global variables
double  E; // Useful for some test functions
struct edgeList edgeKnown;
int edgeListModif;
int edgeListUse;

double	evalNb; // Total number of evaluations
double numMax;

struct bits permut;
	
double pi; // Useful for some test functions
int pivots[sMax]; // Flags to know which particles are pivots
int verbose;

// File(s)
FILE *f_Q; // Matrix for quadratic problems
FILE *f_Qr; // Matrix for random quadratic problems
FILE *f_landscape; // Landscape for Multimodal problem
FILE *f_init; // If the initial position is read from a file
FILE *f_trace;
FILE *f_synth; // Summary of the results

//========================================================================
int main()
{
	struct position best;
	int d;
	double failure;
	double fMean, fMin;
	struct parameter param;
	struct problem pb;
	int r;
	double successRate;
	
//long int num; // For test landscape
//struct position x; // "

	// Define the problem
	pb=pbDefine();		
		
	// Parameters
	param=paramDefine(pb.D);
		
/*		
// Landscape (TEST when numSize=1)
f_trace=fopen("f_trace.txt","w");
for(num=0;num<numMax;num++)
{
	x.bit.size=pb.D;
	x.num.num[0]=num;
	x.bit=numToBit((double)num,pb.D);
	x.f=perf(x,pb);
	fprintf(f_trace,"%f %f\n",x.num.num[0],x.f);			
}		
return 0;
*/
					
	// Optimisation
	fMin=infinity;
	fMean=0;
	failure=0;
	for(r=0;r<pb.runs;r++)
	{
		printf("\n--------------------- run %i",r+1);

		permut=permutBit(pb.D); // Define a random permutation
		printf("\n Permutation: ");
		for (d=0;d<pb.D;d++) printf(" %i", permut.b[d]);

		best=psoBin(pb,param);
		
		printf("\neval %0.f",evalNb);
		if(edgeListUse>0)
		{
			printf("\nKnown edges %i",edgeKnown.size);
			if(verbose>0) edgeListDisplay(edgeKnown);
		}
		positionDisplay(best);
		fMean=fMean+best.f;
		if(best.f<fMin) fMin=best.f;
		if(best.f>pb.objective) failure=failure+1;	
	}
	
	// Statistics 
	fMean=fMean/pb.runs;
	
	// Display statistics
	printf("\n-----------------------");
	successRate=1-failure/pb.runs;
	printf("\nSuccess rate %f",successRate);
	printf("\n Best fitness %f",fMin);
	printf("\n Mean fitness %f",fMean);
	
	return 0;
}
//----------------------------------------------------
struct parameter paramDefine(int D)
{
	struct parameter param;
	double alpha=0.17; //default: 0.17.  Hard coded value. See IJCIR, Special Issue on PSO
											// 2008. Paper "Why does it work"
											// However, for difficult problems, alpha=0.1 seems better
	int K=3; // Hard coded value. 3 is a good compromise. Sometimes, 1 is better
					
	int methodNb=2; // Number of different methods
	param.method[0]=0; 	param.methodProba[0]=0.5;
	param.method[1]=1;	param.methodProba[1]=0.5;


	/*
		0 Low D pseudo-continuous problemt
		1 around the best informant
		-1 Non Assigned
	*/
			 
	param.S=methodNb*(int)(log(1-pow(0.5,1./D))/log(1-alpha)); 
	param.pLink=1-pow(1-(double)1/(param.S),K); 
	
	printf("\nDimension %i, swarm size %i",D,param.S);
	printf("\nLink probability %f",param.pLink);

	
	// For method 0, use Standard PSO 2007 parameters
	param.c0_1=1. / (2 * log ((double) 2)); // 0.721
	param.c0_2= 0.5 + log ((double) 2); // 1.193  
		
	param.numSize=2; // WARNING: D/numSize MUST be an integer
	 								// Define the number of "slices". Each one is a binary number
	 								// In particular, if your computer uses n bits to code a "double", 
						 			// and if D > n, you must have numSize>1
									// Usually, for good performance with strategy 0,
									// numSize should be as small as possible, i.e. int(1+(D-1)/n)
	// Check
	if(D-param.numSize*(D/param.numSize)>0) 
	{
		printf("\n Error. You must have D/param.numSize integer");
		printf("\n Here D=%i, and param.numSize=%i",D,param.numSize);
		exit(1);							 
	}
							 
	// Set global variables
		// User defined
	edgeListUse=0; // 0 = don't use the list, 1 = use the list

	verbose=0;

		// Automatic
	numMax=pow(2,D/param.numSize)-1; // Max numerical value for each binary slice
	return param;
}
//----------------------------------------------------
struct problem pbDefine()
{
	int d;
	int i,j;
	int n1=0; // Useless initial value, just for my stupid compiler
	struct problem pb;
	
	pb.code=6;
	pb.D=100;	// Search space dimension
	pb.evalMax=20000;
	pb.objective=0.000001;
	pb.runs=100;
	
/*
1 Goldberg's order 3 deceptive problem (binary problem) (15,1000)
2 Bipolar order 6  (18,3000)
3 Muhlenbein's order-5 (15,3000)
4 Clerc's order 3 1
5 Clerc's order 3 2
6 Clerc's order 3 (Zebra3) (15,20000)
7 Parabola
8 Using Multimodal bitstring random generator. Use 100D, 20 peaks for test
9 Clerc's Trap
10 Whitney DF2

-1 Quadratic (data on a file)
-2 Quadratic (random matrix)
-8 Multimodal (data on a file)

*/
	pb.density=1.0; // For Quadratic problem -2	
	pb.peaks=20;
	
	switch(pb.code)
	{
		default:
			break;
		
		case -1:
			// Mono objectif xQx problem. Q on a file

  	f_Q=fopen("Q100paper.txt","r"); // Read the matrix
  // 	f_Q=fopen("Q12.txt","r"); // Read the matrix  
   printf("\n Q read from file");
   fscanf(f_Q,"%i",&pb.D);  // WARNING. Read the dimension on the file
   printf("\n Q matrix %ix%i",pb.D,pb.D); // Q is supposed to be integer matrix


  pb.density=0;
   for (i=0;i<pb.D;i++)
		{
    for (d=0;d<pb.D;d++)
      {
        fscanf(f_Q,"%i",&pb.Q[i][d]);
       if(fabs(pb.Q[i][d])>0) pb.density=pb.density+1;
       }
		}
    pb.density=pb.density/(pb.D*pb.D);
    printf("  Density %.2f",pb.density);
    printf("\n Minimum value 0");
    printf("\n");
		break;
		
		case -2:
		// Mono objectif xQx problem. Random Q
  printf("\n Random Q matrix %ix%i",pb.D,pb.D);
  printf("  Density %f",pb.density);
  printf("\n Unknown minimum value.  ...");

  printf("\n");
  f_Qr=fopen("Qr.txt","w"); // Save the matrix
  printf("\n Q saved on Qr.txt");
  fprintf(f_Qr,"%i",pb.D);
  for (i=0;i<pb.D;i++)
  {
    fprintf(f_Qr,"\n");
    for (d=0;d<pb.D;d++)
    {
       if(alea(0,1)<pb.density)
       n1= aleaInteger(-3*pb.D,3*pb.D);
        pb.Q[i][d]=n1;
        fprintf(f_Qr,"%i ",n1);
    }
  }
  pb.code=-1;	
			
			break;
	
	case 8: // Multimodal bitstring random generator
                    // (William Spears)
	 
		f_landscape=fopen("f_landscape.txt","w");

 		for(i = 0; i < pb.peaks; i++)
 	 {
			for(j = 0; j < pb.D; j++)
      {
	        pb.landscape[i][j] = rand()&01;
       }
  	}

		fprintf(f_landscape,"%i %i",pb.D,pb.peaks);  
		for (i=0;i<pb.peaks;i++)
		{
 		 	fprintf(f_landscape,"\n");

	  	for (j=0;j<pb.D;j++)
  		fprintf(f_landscape,"%i ",  pb.landscape[i][j]);
		}
	
	break;
	
	case -8: // Multimodal bitstring random generator
	 
  	f_landscape=fopen("f_landscape_12_20.txt","r");
  	fscanf(f_landscape,"%i %i", &pb.D, &pb.peaks);
  	for(i = 0; i < pb.peaks; i++)
  	{
      for(j = 0; j < pb.D; j++)
      {
	      fscanf(f_landscape,"%i",&pb.landscape[i][j]);
      }
  	}
  	pb.code=8;

	break;
	}
	
	return pb;
}

//----------------------------------------------------
struct position psoBin(struct problem pb, struct parameter param)
{
	double fPrev;
	int improve;
	double pr;
	int rankNew;
	int s,sBest;
	struct swarm S;
	int w;
	
	// Swarm size
	S.size=param.S;
	
	// Initialisations
	edgeKnown.size=0;
	evalNb=0;
	for(s=0;s<S.size;s++)  // For each particle ...
	{	
		// Assign strategy (method) according to the given probabilities
		pr=alea(0,1);
		if(pr<param.methodProba[0]) S.strategy[s]=param.method[0]; 
		else
		{
			 S.strategy[s]=param.method[1];
		}

		// Position
		S.x[s]=positionInit(pb,param); 
		// Velocity
		S.v[s]=velocityInit(pb,param); 
		// Best previous position
		S.xBest[s]=S.x[s]; 
	}
	
	// Links
	linksInit(S.links,S.size,param.pLink);
	
	// Find the best position
	sBest=0;
	for(s=1;s<param.S;s++)
	{
		if(S.xBest[s].f<S.xBest[sBest].f) sBest=s;	
	}
	S.best=S.xBest[sBest];
	S.best.bit.size=pb.D;
	
	printf("\nBest position after init");
	positionDisplay(S.best);
	printf("\n");
	
	// Move
	while(evalNb<=pb.evalMax && S.best.f>pb.objective)
	{
		if(verbose>0) printf("\neval %0.f",evalNb);
		edgeListModif=0;
		fPrev=S.best.f;
		for(s=0;s<param.S;s++) //Iteration. For each particle ...
		{
			S.x[s]=positionUpdate(s,pb,param,S);

			if(S.x[s].f<S.xBest[s].f) 
			{
				S.xBest[s]=S.x[s];
				if(S.xBest[s].f<S.best.f) S.best=S.xBest[s];
			}
		}
		
		if(edgeListUse==1)
		{
			if(edgeListModif==0)
			{
				rankNew=edgeNew(pb,param.numSize); // Choose a new edge
				// Move the worst particle to it
				w=0;
				for(s=1;s<S.size;s++) // Find the worst
				{
					if(S.xBest[s].f>S.xBest[w].f) w=s;	
				}
		
				S.x[w]=edgeKnown.xBin[rankNew];	// Move to the new edge
				//S.xBest[w]=S.x[w];
			}
		}
	
		improve=S.best.f<fPrev;
		if(!improve)
		{
			linksInit(S.links,S.size,param.pLink); // Reinit links
		}
	}
	return S.best;
}
//===========================================================
struct position positionInit(struct problem pb, struct parameter param)
{
	int d;
	int n;
	struct bits slice;
	struct position x;
	x.f=0; // Just for my stupid compiler
	//x.bit.num[d]=0;  // "
							 
	x.bit.size=pb.D;
	// At random
	for(d=0;d<pb.D;d++)
	{
		x.bit.b[d]=aleaInteger(0,1);
	}
	
		// For strategy 0 
	slice.size=pb.D/param.numSize; // Number of bits of each slice
	x.num.size=param.numSize; // Number of slices, and of numerical values
				
	for(n=0;n<param.numSize;n++) // Compute the numerical values
	{
		for(d=0;d<slice.size;d++) slice.b[d]=x.bit.b[d+n*slice.size];
		x.num.num[n]=bitToNum(slice,0,1); // Convert the binary slice into a number
	}
			
	x.f=perf(x,pb);
	
	return x;
}
//===========================================================
struct velocity velocityInit(struct problem pb, struct parameter param)
{
	int d;
	struct velocity v;
	// Null velocity
		v.bit.size=pb.D; // Binary form
		for(d=0;d<pb.D;d++) v.bit.b[d]=0; 

		v.num.size=param.numSize;	 // Numerical form 
		for(d=0;d<param.numSize;d++) v.num.num[d]=0;
	return v;
}
//===========================================================
void linksDisplay(int links[sMax][sMax],int S)
{
	int i,j;
	printf("\nLinks");
	for(i=0;i<S;i++)
	{
		printf("\n");
		for(j=0;j<S;j++)
		{
			printf("%i ",links[i][j]);			
		}	
	}
}
//===========================================================
void linksInit(int links[sMax][sMax],int S, double p)
{
	int i,j;
	if(verbose>0) printf("\nlinksInit");
	for(i=0;i<S;i++)
	{
		for(j=0;j<S;j++)
		{
			if(alea(0,1)<p) links[i][j]=1;
			else links[i][j]=0;
		}
		links[i][i]=1;
	}
		if(verbose>0) linksDisplay(links,S);
}
//===========================================================
struct position positionUpdate
	(int s0,struct problem pb, struct parameter param,struct swarm S)
{
	/*
	WARNING. S may be modified (S.v, in particular)
	*/
	struct bits bit,bitTemp;
	int d;
	int dist;
	int g=0; // Useless initial value, just for my stupid complier
	int i;
	int r;
	int s;
	struct position x;
	x.f=0; 				// Just for my stupid complier				 
	x.bit.size=pb.D;
	x.num.size=param.numSize;
	bit.size=pb.D/param.numSize;
	
	// Find the best informant  (rank g)

	for(s=0;s<S.size;s++) // Find the first
	{
		if(S.links[s][s0]>0) {g=s;break;}
	}
	
	for(s=0;s<S.size;s++) // Find the best
	{
		if(S.links[s][s0]==0) continue;
		if(S.xBest[s].f<S.xBest[g].f) g=s;
	}

	switch(S.strategy[s0])
	{
		case 0: // low D pseudo-continuous. Based on Standard PSO 2007
						// Work on the numerical value of each binary slice
			for(s=0;s<param.numSize;s++)
			{
				S.v[s0].num.num[s]=param.c0_1*S.v[s0].num.num[s]
							+ alea(0,param.c0_2)*(S.xBest[s0].num.num[s]-S.x[s0].num.num[s])
							+ alea(0,param.c0_2)*(S.xBest[g].num.num[s]-S.x[s0].num.num[s]);
				x.num.num[s]=S.x[s0].num.num[s]+S.v[s0].num.num[s]; // Move
		
				// Clamping
				if(x.num.num[s]<0) x.num.num[s]=0;
				else
				{
					if(x.num.num[s]>numMax) x.num.num[s]=numMax;	
				}
				// Keep up-to-date the bit string
				bit=numToBit(x.num.num[s],bit.size);
				for(d=0;d<bit.size;d++) x.bit.b[d+s*bit.size]=bit.b[d];
			}

			// Permut the bit string
			bitTemp=x.bit;
			for(d=0;d<x.bit.size;d++)
			{
				x.bit.b[d]=bitTemp.b[permut.b[d]];
			}

		break;
									 
		default: // Method 1.  Around the best informant 		
  		x=S.xBest[g];
			dist=(int)log(pb.D);
			r=1+aleaInteger(1,dist);

			for(i=0;i<r;i++)
			{
				d=aleaInteger(0,pb.D-1);
				x.bit.b[d]=1-x.bit.b[d];
			}	

			// Keep up-to-date the numerical value(s)
			for(s=0;s<param.numSize;s++)
			{			
				for(d=0;d<bit.size;d++) bit.b[d]=x.bit.b[d+s*bit.size];
				x.num.num[s]=bitToNum (bit,0,1);
			}
		break;	
	}
	x.f=perf(x,pb);
	return x;
}
//===========================================================
void positionDisplay(struct position x)
{
	int d;
	printf("\nf = %f",x.f);
	printf("  x = ");
	for(d=0;d<x.bit.size;d++) printf("%1i",x.bit.b[d]);

}
//===========================================================
double	alea(double a,double b)
{	// rand number (uniform distribution) in [a b]

double 	r;
 r=(double)KissRand()/RAND_MAX_KISS;
// r=(double)rand(); r=r/RAND_MAX;
return a+r*(b-a);
}
//===========================================================
int	aleaInteger(int a,int b)
{// Integer rand number in [a b]
int	ir;
double 	r;
r=alea(0,1); ir=(int)(a+r*(b+1-a)); if (ir>b) ir=b;
return ir;
}
//===========================================================
double	distance(struct position x1, struct position x2)
{
// Hamming distance between two positions
  int	d;
  double dist;
  int n1;
	
  dist=0;
  for (d=0;d<x1.bit.size;d++)
  {
    n1=abs(x1.bit.b[d]- x2.bit.b[d]);
	dist=dist+n1;
  }

  return dist;
}
//===========================================================
int edgeAdd(struct position xBin)
{
	int i;
	int least;
	
	if(edgeKnown.size>=edgeMax)
	{	
		// Replace the least useful
		least=0;
		
		for(i=1;i<edgeKnown.size;i++)
		{
			if(edgeKnown.times[i]<edgeKnown.times[least])
				least=i;	
		}
//	printf("\n replace edge %i",least);		
		edgeKnown.xBin[least]=xBin;
		edgeKnown.times[least]=1;	
	}
	else // Just add the new known edge
	{
//printf("\n add edge");
		least=edgeKnown.size;
		edgeKnown.xBin[least]=xBin;
		edgeKnown.times[least]=1;
		edgeKnown.size=edgeKnown.size+1;
	}
	edgeListModif=1;
	return least;
}
//===========================================================
int edgeFind(struct position xBin)
{
	// return 1 if position/edge xBin is already in the list
	int d;
	int i;
	int OK;
	
	if(edgeKnown.size==0) return -1; // The list is empty
	
	for(i=0;i<edgeKnown.size;i++)
	{	
		OK=1;
		for(d=0;d<xBin.bit.size;d++)
		{
			if(xBin.bit.b[d]!=edgeKnown.xBin[i].bit.b[d]) {OK=0; break;}
		}
		if(OK==1) return i;	// Known, at rank i
	}
	return -1; // Not known
}
// ===========================================================
void edgeListDisplay(struct edgeList edges)
{
	int d,i;
	printf("\n List of the %i known edges:",edges.size);
	for(i=0;i<edges.size;i++)
	{
		printf("\n %i times",edges.times[i]);
		printf(", f  %f",edges.xBin[i].f);
		
		printf(", x =");		
		for(d=0;d<edges.xBin[i].bit.size;d++)
		{
			printf("%i",(int)edges.xBin[i].bit.b[d]);
		}
	}
}
// ===========================================================
int edgeNew(struct problem pb, int numSize)
{
	/* WARNING. This does suppose edgeMax<2^D; i.e. there surely exists an edge
							not already used.
	*/
	int d;
	int s;
	struct bits slice;
	int sliceLength;
	
	int OK=1;
	int rankNew;
	struct position xBin;

	xBin.bit.size=pb.D;

	while(OK==1)
	{
		// Generate an edge at random
		for(d=0;d<pb.D;d++)
		{
			xBin.bit.b[d]=aleaInteger(0,1);
		}	

		// Check if already exists
		OK=edgeFind(xBin);
	
		// if not, add it, and leave
		if(OK!=1)
		{			
			xBin.f=perf(xBin, pb); // Evaluation
			slice.size=numSize; // Numerical value(s) of the binary slice(s)
			xBin.num.size=numSize;
			sliceLength=pb.D/numSize;
			for(s=0;s<numSize;s++)
			{
				for(d=0;d<sliceLength;d++) slice.b[d]=xBin.bit.b[d+s*sliceLength];
				xBin.num.num[s]=bitToNum(slice,0,1);
			}
			// Save it
			rankNew=edgeAdd(xBin);
			if(verbose>1) printf("\n Force a new edge");		
		}	
	}
	return rankNew;
}

//===========================================================
double bitToNum(struct bits x, int d,int code)
{
	// Note: "permut" is a global variable, defined before each run
	int i;
	struct bits xSort;
	double z;

	if(code==0) xSort=x; // Keep the same order
	else // Sort the bit string according to "permut"
	{
		xSort.size=x.size;
		
		for(i=0;i<x.size;i++)
			xSort.b[i]=x.b[permut.b[i]];
	}
	
	z=bitToNum2(xSort,d);
	return z;
}

double bitToNum2(struct bits x, int d)
{
	// Compute the value (base 10) of the bit string
	// WARNING: recursive algorithm
	int D;
  double z;
	D=x.size;
	
  if (d==D-1) return x.b[D-1];
  z=x.b[d]+2*bitToNum2(x,d+1);

  return z;
}
//===========================================================
struct bits numToBit(double a, int size)
{
/* Compute the bit string (truncated at "size")
 corresponding to the number a
*/
	int i,max,rank;
	long int z1,z2;
 struct bits bit;

 for (i=0;i<size;i++) bit.b[i]=0; // Set to zero

 bit.size=size;

 if(a==0) return bit;
 if(a==1) {bit.b[0]=1; return bit; }
 
 z1=a;
 rank=0;
 max=1;
 loop:
 z2=z1/2;
 if(z2==0) {bit.b[rank]=1; goto end;}

if (rank<=size-1)
  {
    if(2*z2<z1) bit.b[rank]=1;
    z1=z2; rank=rank+1; goto loop;
  }

end:
// for (i=0;i<size;i++) printf("%1i",bit.b[size-i-1]);

return bit;
}

//===========================================================
struct bits permutBit(int D)
{
		/* Define a random permutation of the bits that has to be applied
			before converting the bit string into an integer.
			This permutation is valid for a complete run
*/

	int i;
	struct bits permut;
	int r,r1;
	int rank[dMax];
	int rankSize=D;
	permut.size=0;

	for(i=0;i<D;i++) rank[i]=i;

	for (i=0;i<D;i++)
	{	
		// Choose a rank at random, and add it to the permutation
			r=aleaInteger(0,rankSize-1);
			permut.b[permut.size]=rank[r];
			permut.size=permut.size+1;

		// Compact the list of non already used ranks
		if(r<rankSize-1)
		{
			for(r1=r;r1<rankSize-1;r1++)
			{
				rank[r1]=rank[r1+1];
			}
		}
		rankSize=rankSize-1;
	}

	return permut;
}
//===========================================================
double perf(struct position x,struct problem pb)
{
int	bitSum;
struct bits bits;
int D;
int    d;
double  f=0;
double f1;
int		i,j,k;
int known;
int   num;
struct position xBin;

if(edgeListUse==1)
{
	// Check if already known
	known=edgeFind(x);
	// If known, just re-use the fitness
	if(known>=0)
	{
		edgeKnown.times[known]=edgeKnown.times[known]+1; // For information
		return edgeKnown.xBin[known].f;
	}	
}

evalNb=evalNb+1;
D=pb.D;
xBin=x;

switch (pb.code)
{
case 1: // Goldberg's order-3problem:
	// Be sure dimension=3*k
	// The min value is 0  on 1111...111
f=0;
for (d=0;d<D;d=d+3)
{
	bitSum=0;
	for (i=d;i<d+3;i++) bitSum+=x.bit.b[i];

  if (bitSum==0) {f+=0.9; continue;}
 //  if (bitSum==0) {f+=0.1; continue;} 
	if (bitSum==1) {f+=0.6; continue;}
	if (bitSum==2) {f+=0.3; continue;}
	if (bitSum==3) {f+=1.0; continue;}
}

f=D/3-f;
break;


case 2: // Bipolar order-6
        // Min 0 on any position ..6x0..6x1..
 f=0;

for (d=0;d<D-3;d=d+6)
{
	bitSum=0;
	for (i=d;i<d+6;i++) bitSum+=x.bit.b[i];

	if (bitSum==0) {f+=1.0; continue;}
	if (bitSum==1) {f+=0.0; continue;}
	if (bitSum==2) {f+=0.4; continue;}

	if (bitSum==3) {f+=0.8; continue;}
  if (bitSum==4) {f+=0.4; continue;}
  if (bitSum==5) {f+=0.0; continue;}
  if (bitSum==6) {f+=1.0; continue;}
}

f=D/6-f;
 break;


case 3: // Muhlenbein's order 5
        // Min 0 on 000000000...

f=0;
bits.size=5;
for (d=0;d<D-2;d=d+5)
{
	for (i=d;i<d+5;i++) bits.b[i-d]=x.bit.b[i];
   num=(int)bitToNum(bits,0,0);

   if (num==0) {f+=4.0; continue;}
   if (num==1) {f+=3.0; continue;}
   if (num==3) {f+=2.0; continue;}
   if (num==7) {f+=1.0; continue;}
   if (num==31) {f+=3.5; continue;}
}
f=4*D/5-f;
break;

case 4: // Clerc's order 3 problem 1
	// The min value is 0 on 011*
f=0;
for (d=0;d<D;d=d+3)
{
	bitSum=x.bit.b[d+2];
	for (i=d;i<d+2;i++) bitSum+=(int)(x.bit.b[i]*pow(2,2+d-i));
  if (bitSum==0) {f+=0.9; continue;} //0 =000
	if (bitSum==1) {f+=0.6; continue;} //1 =001
	if (bitSum==2) {f+=0.3; continue;} //2 =010
	if (bitSum==3) {f+=1.0; continue;} //3 =011

 	if (bitSum==4) {f+=0.2; continue;} //4 =100

  if (bitSum==5) {f+=0.4; continue;} //5 =101
  if (bitSum==6) {f+=0.6; continue;} //6 =110
  if (bitSum==7) {f+=0.8; continue;} //7 =111

}
f=D/3-f;
break;


case 5: // Clerc's order 3 problem 2
	// The min value is 0 on 001*
f=0;
for (d=0;d<D;d=d+3)
{
	bitSum=x.bit.b[d+2];
	for (i=d;i<d+2;i++) bitSum+=(int)(x.bit.b[i]*pow(2,2+d-i));
  if (bitSum==0) {f+=0.2; continue;} //0 =000
	if (bitSum==1) {f+=1.0; continue;} //1 =001

	if (bitSum==2) {f+=0.3; continue;} //2 =010
	if (bitSum==3) {f+=0.8; continue;} //3 =011
 	if (bitSum==4) {f+=0.6; continue;} //4 =100

  if (bitSum==5) {f+=0.4; continue;} //5 =101
  if (bitSum==6) {f+=0.6; continue;} //6 =110
  if (bitSum==7) {f+=0.9; continue;} //7 =111

}
f=D/3-f;
break;

case 6: // Clerc's order-3 problem 3 (Zebra3)
  // Min 0 on 000111000111...
f=0;
num=0;

for (d=0;d<D;d=d+3)
{
  bitSum=0;
  num=num+1;
	for (i=d;i<d+3;i++) bitSum+=x.bit.b[i];

  if(num%2==0)
  {
    if (bitSum==0) {f+=0.9; continue;}
	  if (bitSum==1) {f+=0.6; continue;}
	  if (bitSum==2) {f+=0.3; continue;}
	  if (bitSum==3) {f+=1.0; continue;}
   }
   else
   {
    if (bitSum==0) {f+=1.0; continue;}
	  if (bitSum==1) {f+=0.3; continue;}
	  if (bitSum==2) {f+=0.6; continue;}
	  if (bitSum==3) {f+=0.9; continue;}
   }
}
f=D/3-f;

break;

case 7: // Parabola
			 // WARNING. We suppose here param.numSize=1
f=x.num.num[0];
 f=f*f;
break;


case 8: // Using Multimodal generated landscape
       f = 0.0;
       for (j = 0; j < pb.peaks; j++)
       {
	        f1 = 0.0;
	        for (k = 0; k < D; k++)
          {
	           if ((int)x.bit.b[k] == pb.landscape[j][k]) f1++;
	        }

	        if (f1 > f) f = f1; 
       }

       f=1-f/(double)D;
break;

case 9: // Clerc's Trap
  // Min 0 at 0000...000
    bitSum=0;
  for (d=0;d<D;d++)   bitSum+=x.bit.b[d];
   if(bitSum==0) f=0;
   else
    f=1.0/bitSum;
  break;

case 10: // Whitney DF2
// Min 0 at 111...1111
f=0;
bits.size=4;
for (d=0;d<D;d=d+4)
{
	for (i=d;i<d+4;i++) bits.b[i-d]=x.bit.b[i];
   num=(int)bitToNum(bits,0,0);

   if (num==15) {f+=0.0; continue;}
   if (num==0) {f+=2.0; continue;}
   if (num==1) {f+=4.0; continue;}

   if (num==2) {f+=6.0; continue;}
   if (num==4) {f+=8.0; continue;}
   if (num==8) {f+=10.0; continue;}
   if (num==3) {f+=12.0; continue;}
   if (num==5) {f+=14.0; continue;}
   if (num==6) {f+=16.0; continue;}
   if (num==9) {f+=18.0; continue;}
   if (num==10) {f+=20.0; continue;}
   if (num==12) {f+=22.0; continue;}

   if (num==14) {f+=24.0; continue;}
   if (num==13) {f+=26.0; continue;}
   if (num==11) {f+=28.0; continue;}
   if (num==7) {f+=30.0; continue;}
}

break;

case -1: // Quadratic problem
		// Q has been read on a file
f=0;
for (i=0;i<D;i++)
for (d=0;d<D;d++)
{
  f=f+pb.Q[i][d]*x.bit.b[i]*x.bit.b[d];
}

f=fabs(D+f); // You may have to change it
break;

}

// Add to the list of known edges
xBin.f=f;
edgeAdd(xBin);

return f;
}

//===========================================================
//Deterministic pseudo random numbers generator KISS.
/*
 the idea is to use simple, fast, individually promising
 generators to get a composite that will be fast, easy to code
 have a very long period and pass all the tests put to it.
 The three components of KISS are
        x(n)=a*x(n-1)+1 mod 2^32
        y(n)=y(n-1)(I+L^13)(I+R^17)(I+L^5),
        z(n)=2*z(n-1)+z(n-2) +carry mod 2^32
 The y's are a shift register sequence on 32bit binary vectors
 period 2^32-1;
 The z's are a simple multiply-with-carry sequence with period
 2^63+2^32-1.  The period of KISS is thus
      2^32*(2^32-1)*(2^63+2^32-1) > 2^127
*/


static ulong kiss_x = 1;
static ulong kiss_y = 2;
static ulong kiss_z = 4;
static ulong kiss_w = 8;

static ulong kiss_carry = 0;
static ulong kiss_k;
static ulong kiss_m;


void KissSeed(ulong seed)
{
    kiss_x = seed | 1;

    kiss_y = seed | 2;
    kiss_z = seed | 4;

    kiss_w = seed | 8;
    kiss_carry = 0;
}

ulong KissRand()
{
    kiss_x = kiss_x * 69069 + 1;
    kiss_y ^= kiss_y << 13;


    kiss_y ^= kiss_y >> 17;
    kiss_y ^= kiss_y << 5;
    kiss_k = (kiss_z >> 2) + (kiss_w >> 3) + (kiss_carry >> 2);
    kiss_m = kiss_w + kiss_w + kiss_z + kiss_carry;
    kiss_z = kiss_w;
    kiss_w = kiss_m;
    kiss_carry = kiss_k >> 30;
    return kiss_x + kiss_y + kiss_w;
}


